/* Silly program to "trivially" (ha!) store uid and gid numbers in correct place.
 * Version 2: Version 1 didn't work with LinuxPPC Release 5
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <pwd.h>
#include <grp.h>
#include <errno.h>

/* Make sure this matches with auto_uids.c */
struct block
   {
   int mrkb;
   int auto_uida;
   int auto_uidd;
   int auto_uidl;
   int auto_uido;
   int auto_uidp;
   int auto_uidq;
   int auto_uidr;
   int auto_uids;
   int auto_gidq;
   int auto_gidn;
   int mrke;
   };

int filebrand( int file, char *filename, struct block * );
int setuser( int *addr, char *username );
int setgroup( int *addr, char *groupname );
void usage();

struct block master_block;

int main( int argc, char **argv )
{
int n;
int fd;

/* Initialize users and groups */
master_block.mrkb = 0x6d726b62; /* Should be "mrkb" */
setuser( &master_block.auto_uida, "alias"  );
setuser( &master_block.auto_uidd, "qmaild" );
setuser( &master_block.auto_uidl, "qmaill" );
setuser( &master_block.auto_uido, "root"   );
setuser( &master_block.auto_uidp, "qmailp" );
setuser( &master_block.auto_uidq, "qmailq" );
setuser( &master_block.auto_uidr, "qmailr" );
setuser( &master_block.auto_uids, "qmails" );

setgroup( &master_block.auto_gidq, "qmail"   );
setgroup( &master_block.auto_gidn, "nofiles" );
master_block.mrke = 0x6d726b65; /* Should be "mrke" */

if ( argc < 2 )
   usage();

for ( n = 1; n < argc; n++ )
   {
   if ( (fd = open( argv[n], O_RDWR )) == -1 )
      {
      fprintf( stderr, "Can't open file %s\n", argv[n] );
      return 1;
      }
   /* fprintf( stderr, "Searching %s\n", argv[n] ); */
   if ( filebrand( fd, argv[n], &master_block ) )
      return 1;
   close( fd );
   }

return 0;
}

void usage( void )
{
fprintf( stderr, "Usage: brand file ...\n" );
exit( 1 );
}

/* Search for beginning and end marker and brand integers in between */
int filebrand( int file, char *filename, struct block *master_block )
{
unsigned char c;
int n;
int pos = 0;
struct block template_block;

/* Go through each character of file, filling in buffer until string terminator
   found or buffer is full */
while ( read( file, &c, sizeof(c) ) )
   {
   pos++;
   if ( c != *((unsigned char *) master_block) )
      continue;

   /* Something found. Read in template block, reset pointer and compare */
   *((unsigned char *) &template_block) = c;
   n = read( file, ((unsigned char *) &template_block) + 1, sizeof(template_block)-1);
   errno = 0;
   lseek( file, pos, SEEK_SET ); /* Reset file back to where we were */
   if ( errno )
      {
      fprintf( stderr, "%s: lseek error.\n" );
      return 1;
      }

   if ( n != sizeof(template_block) - 1 )
      {
      fprintf( stderr, "%s: Error: Short read.\n", filename );
      return 1;
      }

   if ( master_block->mrkb == template_block.mrkb &&
        master_block->mrke == template_block.mrke )
      {
      /* We found the markers, now write out master block to brand file */
      /* fprintf( stderr, "File: %s - position = 0x%0x\n", filename, pos ); */
      if ( write( file, ((char *) master_block) + 1, sizeof(template_block) - 1 ) != (sizeof(template_block) - 1) )
         {
         fprintf( stderr, "%s: Error writing file.\n", filename );
         return 1;
         }

      return 0; /* Everything should have worked */
      }
   }

/* We should have found SOMETHING! */
fprintf( stderr, "%s: Marker not found.\n", filename );
return 1;
}

setuser( int *addr, char *username )
{
struct passwd *myp;

if ( (myp = getpwnam( username )) == NULL )
   {
   fprintf( stderr, "Can't find user %s\n", username );
   exit( 1 );
   }

*addr = (int) (myp->pw_uid);
}

setgroup( int *addr, char *groupname )
{
struct group *myg;

if ( (myg = getgrnam( groupname )) == NULL )
   {
   fprintf( stderr, "Can't find group %s\n", groupname );
   exit( 1 );
   }

*addr = (int) (myg->gr_gid);
}
